@@ -196,6 +196,8 @@ namespace MiniZinc {
           if (BinOp* e_bo = e->dyn_cast<BinOp>()) {
             todo.push_back(e_bo->lhs());
             todo.push_back(e_bo->rhs());
+            for (ExpressionSetIter it = e_bo->ann().begin(); it != e_bo->ann().end(); ++it)
+              run(env, *it);
           } else {
             run(env, e);
           }
@@ -727,6 +729,7 @@ namespace MiniZinc {
             _typeErrors.push_back(TypeError(_env,vdi->loc(),
                                             "type-inst variables not allowed in type-inst for let variable `"+vdi->id()->str().str()+"'"));
           }
+          let.let_orig()[i] = vdi->e();
         }
       }
       Type ty = let.in()->type();
